@@ -4,13 +4,13 @@ extern int g_connection;
 static DISPLAY_EVENT_HANDLER(display_handler)
 {
     if (flags & kCGDisplayAddFlag) {
-        event_loop_post(&g_event_loop, DISPLAY_ADDED, (void *)(intptr_t) did, 0, NULL);
+        event_loop_post(&g_event_loop, DISPLAY_ADDED, (void *)(intptr_t) did, 0);
     } else if (flags & kCGDisplayRemoveFlag) {
-        event_loop_post(&g_event_loop, DISPLAY_REMOVED, (void *)(intptr_t) did, 0, NULL);
+        event_loop_post(&g_event_loop, DISPLAY_REMOVED, (void *)(intptr_t) did, 0);
     } else if (flags & kCGDisplayMovedFlag) {
-        event_loop_post(&g_event_loop, DISPLAY_MOVED, (void *)(intptr_t) did, 0, NULL);
+        event_loop_post(&g_event_loop, DISPLAY_MOVED, (void *)(intptr_t) did, 0);
     } else if (flags & kCGDisplayDesktopShapeChangedFlag) {
-        event_loop_post(&g_event_loop, DISPLAY_RESIZED, (void *)(intptr_t) did, 0, NULL);
+        event_loop_post(&g_event_loop, DISPLAY_RESIZED, (void *)(intptr_t) did, 0);
     }
 }
 
@@ -196,7 +196,7 @@ uint64_t *display_space_list(uint32_t did, int *count)
         CFArrayRef spaces_ref = CFDictionaryGetValue(display_ref, CFSTR("Spaces"));
         int spaces_count = CFArrayGetCount(spaces_ref);
 
-        space_list = ts_alloc_aligned(sizeof(uint64_t), spaces_count);
+        space_list = ts_alloc_list(uint64_t, spaces_count);
         *count = spaces_count;
 
         for (int j = 0; j < spaces_count; ++j) {
