@@ -259,6 +259,16 @@ void event_signal_push(enum signal_type type, void *context)
         snprintf(es->arg_name[1],  arg_size, "%s", "YABAI_RECENT_DISPLAY_ID");
         snprintf(es->arg_value[1], arg_size, "%d", g_display_manager.last_display_id);
     } break;
+    case SIGNAL_MISSION_CONTROL_ENTER:
+    case SIGNAL_MISSION_CONTROL_EXIT: {
+        enum mission_control_mode mode = (enum mission_control_mode)(uintptr_t) context;
+
+        es->arg_name[0]  = ts_alloc_unaligned(arg_size);
+        es->arg_value[0] = ts_alloc_unaligned(arg_size);
+
+        snprintf(es->arg_name[0],  arg_size, "%s", "YABAI_MISSION_CONTROL_MODE");
+        snprintf(es->arg_value[0], arg_size, "%s", mission_control_mode_str[mode]);
+    } break;
     }
 }
 
