@@ -26,7 +26,38 @@ static const char *process_name_blacklist[] =
     "siriactionsd",
     "chronod",
     "universalaccessd",
-    "softwareupdated"
+    "softwareupdated",
+    "Notification Centre",
+    "CoreServicesUIAgent",
+    "SystemUIServer",
+    "Control Centre",
+    "BackgroundTaskManagementAgent",
+    "Wallpaper",
+    "talagent",
+    "CoreLocationAgent",
+    "Single Sign-On",
+    "com.apple.PressAndHold",
+    "Wi-Fi",
+    "Keychain Circle Notification",
+    "TextInputMenuAgent",
+    "TextInputSwitcher",
+    "AirPlayUIAgent",
+    "AppSSODaemon",
+    "SoftwareUpdateNotificationManager",
+    "coreautha",
+    "OSDUIHelper",
+    "PowerChime",
+    "nbagent",
+    "studentd",
+    "Family",
+    "Spotlight",
+    "skhd",
+    "yabai",
+    "sketchybar",
+    "Übersicht",
+    "Slack Helper (Plugin)",
+    "Google Chrome Helper (Plugin)",
+    "osascript"
 };
 
 #pragma clang diagnostic push
