@@ -100,19 +100,47 @@ void window_manager_query_windows_for_displays(FILE *rsp)
     window_manager_query_windows_for_spaces(rsp, space_list, space_count);
 }
 
-void window_manager_apply_rule_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule *rule)
+void window_manager_apply_manage_rule_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule *rule, char *window_title, char *window_role, char *window_subrole)
 {
     int regex_match_app = rule->app_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
     if (regex_match(rule->app_regex_valid,   &rule->app_regex,   window->application->name) == regex_match_app) return;
 
     int regex_match_title = rule->title_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
-    if (regex_match(rule->title_regex_valid, &rule->title_regex, window_title_ts(window)) == regex_match_title) return;
+    if (regex_match(rule->title_regex_valid, &rule->title_regex, window_title) == regex_match_title) return;
 
     int regex_match_role = rule->role_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
-    if (regex_match(rule->role_regex_valid, &rule->role_regex, window_role_ts(window)) == regex_match_role) return;
+    if (regex_match(rule->role_regex_valid, &rule->role_regex, window_role) == regex_match_role) return;
 
     int regex_match_subrole = rule->subrole_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
-    if (regex_match(rule->subrole_regex_valid, &rule->subrole_regex, window_subrole_ts(window)) == regex_match_subrole) return;
+    if (regex_match(rule->subrole_regex_valid, &rule->subrole_regex, window_subrole) == regex_match_subrole) return;
+
+    if (rule->manage == RULE_PROP_ON) {
+        window_rule_set_flag(window, WINDOW_RULE_MANAGED);
+        window_manager_make_window_floating(sm, wm, window, false);
+    } else if (rule->manage == RULE_PROP_OFF) {
+        window_rule_clear_flag(window, WINDOW_RULE_MANAGED);
+        window_manager_make_window_floating(sm, wm, window, true);
+    }
+}
+
+void window_manager_apply_rule_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule *rule, char *window_title, char *window_role, char *window_subrole)
+{
+    int regex_match_app = rule->app_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
+    if (regex_match(rule->app_regex_valid,   &rule->app_regex,   window->application->name) == regex_match_app) return;
+
+    int regex_match_title = rule->title_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
+    if (regex_match(rule->title_regex_valid, &rule->title_regex, window_title) == regex_match_title) return;
+
+    int regex_match_role = rule->role_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
+    if (regex_match(rule->role_regex_valid, &rule->role_regex, window_role) == regex_match_role) return;
+
+    int regex_match_subrole = rule->subrole_regex_exclude ? REGEX_MATCH_YES : REGEX_MATCH_NO;
+    if (regex_match(rule->subrole_regex_valid, &rule->subrole_regex, window_subrole) == regex_match_subrole) return;
+
+    if (!window_rule_check_flag(window, WINDOW_RULE_MANAGED)) {
+        if (!rule->role_regex_valid    && !string_equals(window_role   , "AXWindow"))         return;
+        if (!rule->subrole_regex_valid && !string_equals(window_subrole, "AXStandardWindow")) return;
+    }
 
     if (rule->sid || rule->did) {
         if (!window_is_fullscreen(window) && !space_is_fullscreen(window_space(window))) {
@@ -124,14 +152,6 @@ void window_manager_apply_rule_to_window(struct space_manager *sm, struct window
         }
     }
 
-    if (rule->manage == RULE_PROP_ON) {
-        window_rule_set_flag(window, WINDOW_RULE_MANAGED);
-        window_manager_make_window_floating(sm, wm, window, false);
-    } else if (rule->manage == RULE_PROP_OFF) {
-        window_rule_clear_flag(window, WINDOW_RULE_MANAGED);
-        window_manager_make_window_floating(sm, wm, window, true);
-    }
-
     if (rule->sticky == RULE_PROP_ON) {
         window_manager_make_window_sticky(sm, wm, window, true);
     } else if (rule->sticky == RULE_PROP_OFF) {
@@ -165,10 +185,17 @@ void window_manager_apply_rule_to_window(struct space_manager *sm, struct window
     }
 }
 
-void window_manager_apply_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window)
+void window_manager_apply_manage_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole)
 {
     for (int i = 0; i < buf_len(wm->rules); ++i) {
-        window_manager_apply_rule_to_window(sm, wm, window, &wm->rules[i]);
+        window_manager_apply_manage_rule_to_window(sm, wm, window, &wm->rules[i], window_title, window_role, window_subrole);
+    }
+}
+
+void window_manager_apply_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole)
+{
+    for (int i = 0; i < buf_len(wm->rules); ++i) {
+        window_manager_apply_rule_to_window(sm, wm, window, &wm->rules[i], window_title, window_role, window_subrole);
     }
 }
 
@@ -655,9 +682,26 @@ next:
     }
 }
 
+void window_manager_adjust_layer(struct window *window, int layer)
+{
+    if (window->layer != LAYER_AUTO) return;
+
+    scripting_addition_set_layer(window->id, layer);
+}
+
 bool window_manager_set_window_layer(struct window *window, int layer)
 {
-    bool result = scripting_addition_set_layer(window->id, layer);
+    int parent_layer = layer;
+    int child_layer = layer;
+
+    if (layer == LAYER_AUTO) {
+        parent_layer = window_manager_find_managed_window(&g_window_manager, window) ? LAYER_BELOW : LAYER_NORMAL;
+        child_layer = LAYER_NORMAL;
+    }
+
+    window->layer = layer;
+    bool result = scripting_addition_set_layer(window->id, parent_layer);
+    if (!result) return false;
 
     CFArrayRef window_list = SLSCopyAssociatedWindows(g_connection, window->id);
     if (!window_list) return result;
@@ -683,7 +727,7 @@ bool window_manager_set_window_layer(struct window *window, int layer)
     for (int i = 0; i < check_count; ++i) {
         for (int j = 0; j < window_count; ++j) {
             if (parent_list[j] != check_list[i]) continue;
-            scripting_addition_set_layer(child_list[j], layer);
+            scripting_addition_set_layer(child_list[j], child_layer);
             check_list[check_count++] = child_list[j];
         }
     }
@@ -1230,6 +1274,11 @@ struct window *window_manager_create_and_add_window(struct space_manager *sm, st
 {
     struct window *window = window_create(application, window_ref, window_id);
 
+    char *window_title = window_title_ts(window);
+    char *window_role = window_role_ts(window);
+    char *window_subrole = window_subrole_ts(window);
+    debug("%s:%d %s - %s (%s:%s)\n", __FUNCTION__, window->id, window->application->name, window_title, window_role, window_subrole);
+
     if (window_is_unknown(window)) {
         debug("%s: ignoring AXUnknown window %s %d\n", __FUNCTION__, window->application->name, window->id);
         window_manager_remove_lost_focused_event(wm, window->id);
@@ -1244,8 +1293,41 @@ struct window *window_manager_create_and_add_window(struct space_manager *sm, st
         return NULL;
     }
 
-    window_manager_purify_window(wm, window);
-    window_manager_set_window_opacity(wm, window, wm->normal_window_opacity);
+    if (window_is_group(window)) {
+        debug("%s: ignoring AXGroup window %s %d\n", __FUNCTION__, window->application->name, window->id);
+        window_manager_remove_lost_focused_event(wm, window->id);
+        window_destroy(window);
+        return NULL;
+    }
+
+    //
+    // NOTE(koekeishiya): A lot of windows misreport their accessibility role, so we allow the user
+    // to specify rules to make sure that we do in fact manage these windows properly.
+    //
+    // This part of the rule must be applied at this stage (prior to other rule properties), and if
+    // no such rule matches this window, it will be ignored if it does not have a role of kAXWindowRole.
+    //
+
+    window_manager_apply_manage_rules_to_window(sm, wm, window, window_title, window_role, window_subrole);
+
+    if (!window_is_really_a_window(window) && !window_rule_check_flag(window, WINDOW_RULE_MANAGED)) {
+        debug("%s ignoring incorrectly marked window %s %d\n", __FUNCTION__, window->application->name, window->id);
+
+        //
+        // NOTE(koekeishiya): Print window information when debug_output is enabled.
+        // Useful for identifying and creating rules if this window should in fact be managed.
+        //
+
+        if (g_verbose) {
+            fprintf(stdout, "window info: \n");
+            window_serialize(stdout, window);
+            fprintf(stdout, "\n");
+        }
+
+        window_manager_remove_lost_focused_event(wm, window->id);
+        window_destroy(window);
+        return NULL;
+    }
 
     if (!window_observe(window)) {
         debug("%s: could not observe %s %d\n", __FUNCTION__, window->application->name, window->id);
@@ -1260,9 +1342,10 @@ struct window *window_manager_create_and_add_window(struct space_manager *sm, st
         window_manager_remove_lost_focused_event(wm, window->id);
     }
 
-    debug("%s:%d %s - %s\n", __FUNCTION__, window->id, window->application->name, window_title_ts(window));
+    window_manager_apply_rules_to_window(sm, wm, window, window_title, window_role, window_subrole);
     window_manager_add_window(wm, window);
-    window_manager_apply_rules_to_window(sm, wm, window);
+    window_manager_purify_window(wm, window);
+    window_manager_set_window_opacity(wm, window, wm->normal_window_opacity);
 
     if ((!application->is_hidden) &&
         (!window_check_flag(window, WINDOW_MINIMIZE)) &&
@@ -1465,7 +1548,7 @@ enum window_op_error window_manager_stack_window(struct space_manager *sm, struc
 
     view_stack_window_node(a_view, a_node, b);
     window_manager_add_managed_window(wm, b, a_view);
-    scripting_addition_set_layer(b->id, LAYER_BELOW);
+    window_manager_adjust_layer(b, LAYER_BELOW);
     scripting_addition_order_window(b->id, 1, a_node->window_order[1]);
 
     struct area area = a_node->zoom ? a_node->zoom->area : a_node->area;
@@ -1994,7 +2077,7 @@ static void window_manager_validate_windows_on_space(struct space_manager *sm, s
             //
 
             view_remove_window_node(view, window);
-            scripting_addition_set_layer(window->id, LAYER_NORMAL);
+            window_manager_adjust_layer(window, LAYER_NORMAL);
             window_manager_remove_managed_window(wm, window->id);
             window_manager_purify_window(wm, window);
 
@@ -2023,7 +2106,7 @@ static void window_manager_check_for_windows_on_space(struct space_manager *sm,
             //
 
             view_remove_window_node(existing_view, window);
-            scripting_addition_set_layer(window->id, LAYER_NORMAL);
+            window_manager_adjust_layer(window, LAYER_NORMAL);
             window_manager_remove_managed_window(wm, window->id);
             window_manager_purify_window(wm, window);
             existing_view->is_dirty = true;
@@ -2042,7 +2125,7 @@ static void window_manager_check_for_windows_on_space(struct space_manager *sm,
             //
 
             view_add_window_node(view, window);
-            scripting_addition_set_layer(window->id, LAYER_BELOW);
+            window_manager_adjust_layer(window, LAYER_BELOW);
             window_manager_add_managed_window(wm, window, view);
             view->is_dirty = true;
         }
